==> resh-5.0.0.16-py3-none-any.whl: resh/domains/module.py <==
import os
from time import sleep

from resh.domains._domain import Domain
from resh.protocols.can_utils.can_packet import CanDump
from resh.protocols.can_utils.interface import CanConnection
from resh.protocols.can_utils.rapida import commands as rcommands, responses as rresp
from resh.protocols.can_utils.rapida_packet import RapidaSend, RapidaDump
from resh.utils import groups, monitor_filter, tools, compat, settings
from resh.utils.const import *
from resh.utils.errors import CustomError, DeviceNotAnswered, ModuleConstructError, BadParameter, PacketParseError, \
    CanOpenError
from resh.utils.logger import log

"""Loader"""
BL_INFO = 0xB0       # запрос информации о загрузчике
LOAD_INIT = 0xB2     # инициация загрузки

DALI_TYPE = 1
MODBUS_TYPE = 2


@compat.add_metaclass(compat.DomainTracker)
class Module(Domain):
    def_params = {MODULE: 2}
    domain_name = MODULE_DOMAIN
    _module_type = 1

    def _send_raw(self, frame: bytes):
        if not isinstance(frame, (bytes, bytearray)):
            try:
                frame = bytes([int(b) for b in frame])
            except:
                raise BadParameter()

        message = RapidaSend(self._module_type, self.module, frame)
        return message.send_raw()

    def _send_cmd(self, cmd, *params, **kwargs) -> rresp.Response:
        try:
            _cmd = cmd(*[int(p) for p in params])
        except:
            raise BadParameter()

        pack = RapidaSend(self._module_type, self.module, _cmd.frame, **kwargs)
        return pack.send(_cmd.is_query)

    def _send(self, frame=None, need_answer=False, **kwargs):
        try:
            if frame is None:
                frame = []
                while self.has_next:
                    frame.append(int(self.next_param))

            elif isinstance(frame, int):
                frame = [frame]
            else:
                frame = list(frame)

            frame = bytes(frame)
        except:
            raise BadParameter()

        message = RapidaSend(self._module_type, self.module, frame, **kwargs)
        return message.send(need_answer=need_answer)

    def _listen_for(self, cmd_byte, timeout=0.05):
        message = RapidaSend(self._module_type, self.module, [cmd_byte-1], answer_timeout=timeout)
        message.start_answer_timer()
        return message.receive()


"""https://kiwi.throne.systems/index.php/Tron/Strata/Hardware/Rapida/Resh/Command/Rapida#Base"""


class raw_command(Module, groups.Base):
    def_params = {CLASS: 1, MODULE_0_31: 2}
    opt_params = {BYTES_ARR: '1 0xff 255'}

    def exec(self):
        try:
            self._module_type = int(self.next_param)
        except:
            raise BadParameter()
        self._send()


class raw_query(Module, groups.Base):
    def_params = {CLASS: 1, MODULE_0_31: 2}
    opt_params = {BYTES_ARR: '1 0xff 255'}

    def exec(self):
        try:
            self._module_type = int(self.next_param)
        except:
            raise BadParameter()
        return self._send(need_answer=True)


"""https://kiwi.throne.systems/index.php/Tron/Strata/Hardware/Rapida/Resh/Command/Rapida#Standard"""


class readdress(Module, groups.Common):
    def_params = {}

    def exec(self):
        self._module = 0
        self._send_cmd(rcommands.Reboot)
        sleep(0.05)
        try:
            self._send_cmd(rcommands.CommitAddresses, answer_timeout=5)
        except DeviceNotAnswered as ex:
            stack = ex.stack
            if 0 not in stack:
                raise ModuleConstructError('Expected answer from 01F is absent!')
            if len(stack) > 1:
                stack.pop(0)
                raise ModuleConstructError('Unexpected answer from modules {}'.format(stack.keys()))
            stack = stack.pop(0)
            unaddressed = []
            for dump in stack:
                if isinstance(dump.payload.command, rresp.UnaddressedNotify):
                    if dump.data not in unaddressed:
                        unaddressed.append(dump.data)
            if len(unaddressed) == 0:
                raise ModuleConstructError('no module has responded to 0xAD')

            expect_count = len(unaddressed)

            try:
                self._send_cmd(rcommands.RunAutoAddressing)
            except DeviceNotAnswered as ex:
                stack = ex.stack
                if 7 not in stack:
                    raise ModuleConstructError('0xAA command not answered')

                addressed = []
                for dump in stack[7]:
                    if isinstance(dump.payload.command, rcommands.NextAddress):
                        addressed.append(dump.data[0] - 1)
                if len(addressed) == 0:
                    raise ModuleConstructError('No module answered on 0xAA')
                if addressed.count(1) > 1:
                    raise ModuleConstructError('One of the not outer slots is empty')
                if expect_count != len(addressed):
                    raise ModuleConstructError('{} modules found but {} addressed. Wtf?'.format(expect_count, len(addressed)))

                try:
                    self._send_cmd(rcommands.CommitAddresses)
                except DeviceNotAnswered:
                    pass

                return len(addressed)


class find(Module, groups.Common):
    def_params = {}

    def exec(self):
        self._module = 0
        res = []
        try:
            self._send_cmd(rcommands.QueryUidMin)
        except DeviceNotAnswered as ex:
            if ex.stack:
                for m_id, dumps in ex.stack.copy().items():
                    for dump in dumps:
                        if isinstance(dump, RapidaDump):
                            res.append(
                                dict(address=m_id,
                                     type=dump.module_type,
                                     desc=dump.module_desc)
                            )
                            break
        return sorted(res, key=lambda item: item['address'])


class set_address(Module, groups.Common):
    opt_params = {'new_address': 4}

    def exec(self):
        new_address = self.next_param
        self._send_cmd(rcommands.SetAddress, new_address)
        self._module = new_address
        self._listen_for(0x01, 5)


class info(Module, groups.Common):
    def exec(self):
        res = version(*self._args).exec()
        res['serial'] = serial(*self._args).exec()
        return res


class _update_firmware(Module, groups.Common):
    opt_params = {FILE: 'path/to/file.ext'}

    def exec(self):
        path = self.next_param
        if not os.path.exists(path):
            raise CustomError('File "{}" not exist!'.format(path))
        if not os.path.isfile(path):
            raise CustomError('"{}" is not a file!'.format(path))

        self._send_cmd(rcommands.Reboot)
        self._send_cmd(rcommands.InitLoad, answer_timeout=5)
        with open(path, 'rb') as file:
            chunk = file.read(8)
            while chunk:
                self._send_raw(chunk)
                chunk = file.read(8)

        self._listen_for(0xb5, 5)
        self._listen_for(0xba, 5)
        self._listen_for(0xb1, 5)
        self._listen_for(0x01, 5)


class update_firmware(_update_firmware):
    pass


class update_power_firmware(_update_firmware):
    def_params = {}
    _module_type = 31
    _module = 30


class reboot(Module, groups.Common):
    def exec(self): self._send_cmd(rcommands.Reboot)


class version(Module, groups.Common):
    def exec(self):
        val = self._send_cmd(rcommands.QueryVersion).value
        return dict(hardwareVersion=val['hv'],
                    firmwareVersion=val['fv'],
                    bootloaderVersion=val['bv'])


class serial(Module, groups.Common):
    def exec(self):
        serial = self._send_cmd(rcommands.QueryUidMin).data
        serial += self._send_cmd(rcommands.QueryUidMax).data
        return ''.join(hex(b)[2:].rjust(2, '0').upper() for b in serial)


""" Dali """


class __Dali(Module, groups.Dali):
    _module_type = DALI_TYPE


class dali_status(__Dali):
    def exec(self):
        r = self._send_cmd(rcommands.QueryLineStatus)
        r = bin(r.data[0])[:1:-1].ljust(8, '0')
        result = [dict(id=i + 1, powered=r[i] == '1') for i in range(0, len(r))]
        return result


""" Modbus """


class __Modbus(Module, groups.Modbus):
    def_params = {MODULE: 1, CHANNEL: 2}
    _module_type = MODBUS_TYPE


class modbus_channel_status(__Modbus):
    def exec(self): return self._send_cmd(rcommands.QueryChannelConfig, self.channel).value


class modbus_reset_channel(__Modbus):
    def exec(self): return self._send_cmd(rcommands.ResetChannel, self.channel)


class modbus_set_channel_conf(__Modbus):
    opt_params = {BAUD: 2400, PARITY: 'odd', STOP_BITS: 2}

    def exec(self):
        try:
            baud = rresp.ChannelConfig._values["baud"].index(self.next_param)
            parity = rresp.ChannelConfig._values["parity"].index(self.next_param)
            stop = rresp.ChannelConfig._values["stop"].index(self.next_param)
        except:
            raise BadParameter()

        return self._send_cmd(rcommands.SetChannelConfig, self.channel, baud, parity, stop)


""" Wizards """


class monitor(Module, groups.Wizards):
    def_params = {}
    var_params = {MODULE: 1, CHANNEL: 1, DEVICE_ADDR: 'D3,-D4', INSTANCE: 'IT4', NAME: 'event'}

    @classmethod
    def example(cls):
        return ' '.join([cls.domain_name, cls.__name__] +
                        [f'{cls.def_params[p]}' for p in cls.def_params] +
                        [f'{cls.opt_params[p]}' for p in cls.opt_params] +
                        [f'{cls.var_params[p]}' for p in cls.var_params]).lower() + \
               '\n' + cls.loc(cls.domain_name, cls.__name__, NAMED_PARAMS_DESC) + \
               '\n' + cls.loc(cls.domain_name, cls.__name__, EXCLUDE_PARAMS_DESC) + \
               '\n' + cls.loc(cls.domain_name, cls.__name__, BROADCAST_VALUE)

    def _prepare(self):
        self.filter = monitor_filter.Filter(self._args)
        self.can = CanConnection(interface=settings.conf.interface)

    def exec(self):
        try:
            self._prepare()
        except OSError as ex:
            raise CanOpenError(reason=str(ex))

        try:
            for dump in self.can.bus:
                dump = CanDump.can_dump(dump)
                try:
                    payload = dump.payload
                    if self.filter.check(payload):
                        log.monitor(payload)
                except BaseException as ex:
                    try:
                        log.exception(PacketParseError(dump.payload.header_str() + ' ' + str(ex)))
                    except:
                        log.exception(PacketParseError(tools.bytes2str(dump) + ' ' + str(ex)))
        except:
            return

    def set_break(self, sig, params):
        super(monitor, self).set_break(sig, params)
        self.can.stop()

==> resh-5.1.0.9-py3-none-any.whl: resh/domains/module.py <==
import os
from time import sleep

from resh.domains._domain import Domain
from resh.protocols.can_utils.can_packet import CanDump
from resh.protocols.can_utils.interface import CanConnection
from resh.protocols.can_utils.rapida import commands as rcommands, responses as rresp
from resh.protocols.can_utils.rapida_packet import RapidaSend, RapidaDump
from resh.utils import groups, monitor_filter, tools, compat, settings
from resh.utils.const import *
from resh.utils.errors import CustomError, DeviceNotAnswered, ModuleConstructError, BadParameter, PacketParseError, \
    CanOpenError
from resh.utils.logger import log

"""Loader"""
BL_INFO = 0xB0       # запрос информации о загрузчике
LOAD_INIT = 0xB2     # инициация загрузки

DALI_TYPE = 1
MODBUS_TYPE = 2


@compat.add_metaclass(compat.DomainTracker)
class Module(Domain):
    def_params = {MODULE: 2}
    domain_name = MODULE_DOMAIN
    _module_type = 1

    def _send_raw(self, frame: bytes):
        if not isinstance(frame, (bytes, bytearray)):
            try:
                frame = bytes([int(b) for b in frame])
            except:
                raise BadParameter()

        message = RapidaSend(self._module_type, self.module, frame)
        return message.send_raw()

    def _send_cmd(self, cmd, *params, filters=None, **kwargs) -> rresp.Response:
        try:
            _cmd = cmd(*[int(p) for p in params])
        except:
            raise BadParameter()

        pack = RapidaSend(self._module_type, self.module, _cmd.frame, **kwargs)
        if filters is not None:
            pack.bus.set_filters(filters=filters)
        return pack.send(_cmd.is_query)

    def _send(self, frame=None, need_answer=False, **kwargs):
        try:
            if frame is None:
                frame = []
                while self.has_next:
                    frame.append(int(self.next_param))

            elif isinstance(frame, int):
                frame = [frame]
            else:
                frame = list(frame)

            frame = bytes(frame)
        except:
            raise BadParameter()

        message = RapidaSend(self._module_type, self.module, frame, **kwargs)
        return message.send(need_answer=need_answer)

    def _listen_for(self, cmd_byte, timeout=0.05):
        message = RapidaSend(self._module_type, self.module, [cmd_byte-1], answer_timeout=timeout)
        message.start_answer_timer()
        return message.receive()


"""https://kiwi.throne.systems/index.php/Tron/Strata/Hardware/Rapida/Resh/Command/Rapida#Base"""


class raw_command(Module, groups.Base):
    def_params = {CLASS: 1, MODULE_0_31: 2}
    opt_params = {BYTES_ARR: '1 0xff 255'}

    def exec(self):
        try:
            self._module_type = int(self.next_param)
        except:
            raise BadParameter()
        self._send()


class raw_query(Module, groups.Base):
    def_params = {CLASS: 1, MODULE_0_31: 2}
    opt_params = {BYTES_ARR: '1 0xff 255'}

    def exec(self):
        try:
            self._module_type = int(self.next_param)
        except:
            raise BadParameter()
        return self._send(need_answer=True)


"""https://kiwi.throne.systems/index.php/Tron/Strata/Hardware/Rapida/Resh/Command/Rapida#Standard"""


class readdress(Module, groups.Common):
    def_params = {}
    var_params = {FORCED: ''}

    def exec(self):
        self._module = 0
        self._send_cmd(rcommands.Reboot)
        sleep(0.05)
        addressed = []
        forced = False

        if FORCED in self._args:
            forced = True
            self._args.remove(FORCED)

        try:
            self._send_cmd(rcommands.CommitAddresses, answer_timeout=5)
        except DeviceNotAnswered as ex:
            if forced:
                try:
                    self._send_cmd(rcommands.RunAutoAddressing, answer_timeout=5)
                except DeviceNotAnswered:
                    pass
                finally:
                    try:
                        self._send_cmd(rcommands.CommitAddresses)
                    except DeviceNotAnswered:
                        pass
            else:
                stack = ex.stack
                if stack is None or 0 not in stack:
                    raise ModuleConstructError('Expected answer from 01F is absent!')
                if len(stack) > 1:
                    stack.pop(0)
                    raise ModuleConstructError('Unexpected answer from modules {}'.format(stack.keys()))
                stack = stack.pop(0)
                unaddressed = []
                for dump in stack:
                    if isinstance(dump.payload.command, rresp.UnaddressedNotify):
                        if dump.data not in unaddressed:
                            unaddressed.append(dump.data)
                if len(unaddressed) == 0:
                    raise ModuleConstructError('no module has responded to 0xAD')

                expect_count = len(unaddressed)

                try:
                    self._send_cmd(rcommands.RunAutoAddressing, filters=RapidaSend.filters(0x3e0), answer_timeout=5)
                except DeviceNotAnswered as ex:
                    stack = ex.stack
                    if stack is None:
                        stack = {}
                    if RapidaSend.CONTROLLER_ID not in stack:
                        print('RunAutoAddressing(0xAA) command not answered')
                    else:
                        for dump in stack[RapidaSend.CONTROLLER_ID]:
                            if isinstance(dump.payload.command, rcommands.NextAddress):
                                addressed.append(dump.data[0] - 1)
                        if len(addressed) == 0:
                            print('No module answered on 0xAA')
                        elif addressed.count(1) > 1:
                            print('One of the not outer slots is empty')
                        elif expect_count != len(addressed):
                            print('{} modules found but {} addressed. Wtf?'.format(expect_count, len(addressed)))
                finally:
                    try:
                        self._send_cmd(rcommands.CommitAddresses)
                    except DeviceNotAnswered:
                        pass

                return len(addressed)


class find(Module, groups.Common):
    def_params = {}

    def exec(self):
        self._module = 0
        res = []
        try:
            self._send_cmd(rcommands.QueryUidMin, filters=[])
        except DeviceNotAnswered as ex:
            if ex.stack:
                for m_id, dumps in ex.stack.copy().items():
                    for dump in dumps:
                        if isinstance(dump, RapidaDump):
                            res.append(
                                dict(address=m_id,
                                     type=dump.module_type,
                                     name=dump.module_name,
                                     desc=dump.module_desc)
                            )
                            break
        return sorted(res, key=lambda item: item['address'])


class set_address(Module, groups.Common):
    opt_params = {'new_address': 4}

    def exec(self):
        new_address = self.next_param
        self._send_cmd(rcommands.SetAddress, new_address)
        self._module = new_address
        self._listen_for(0x01, 5)


class info(Module, groups.Common):
    def exec(self):
        res = version(*self._args).exec()
        res['serial'] = serial(*self._args).exec()
        return res


class _update_firmware(Module, groups.Common):
    opt_params = {FILE: 'path/to/file.ext'}

    def exec(self):
        path = self.next_param
        if not os.path.exists(path):
            raise CustomError('File "{}" not exist!'.format(path))
        if not os.path.isfile(path):
            raise CustomError('"{}" is not a file!'.format(path))

        self._send_cmd(rcommands.Reboot)
        self._send_cmd(rcommands.InitLoad, answer_timeout=5)
        with open(path, 'rb') as file:
            chunk = file.read(8)
            while chunk:
                self._send_raw(chunk)
                chunk = file.read(8)

        self._listen_for(0xb5, 5)
        self._listen_for(0xba, 5)
        self._listen_for(0xb1, 5)
        self._listen_for(0x01, 5)


class update_firmware(_update_firmware):
    pass


class update_power_firmware(_update_firmware):
    def_params = {}
    _module_type = 31
    _module = 30


class reboot(Module, groups.Common):
    def exec(self): self._send_cmd(rcommands.Reboot)


class version(Module, groups.Common):
    def exec(self):
        val = self._send_cmd(rcommands.QueryVersion).value
        return dict(hardwareVersion=val['hv'],
                    firmwareVersion=val['fv'],
                    bootloaderVersion=val['bv'])


class serial(Module, groups.Common):
    def exec(self):
        serial = self._send_cmd(rcommands.QueryUidMin).data
        serial += self._send_cmd(rcommands.QueryUidMax).data
        return ''.join(hex(b)[2:].rjust(2, '0').upper() for b in serial)


""" Dali """


class __Dali(Module, groups.Dali):
    _module_type = DALI_TYPE


class dali_status(__Dali):
    def exec(self):
        r = self._send_cmd(rcommands.QueryLineStatus)
        r = bin(r.data[0])[:1:-1].ljust(8, '0')
        result = [dict(id=i + 1, powered=r[i] == '1') for i in range(0, len(r))]
        return result


""" Modbus """


class __Modbus(Module, groups.Modbus):
    def_params = {MODULE: 1, CHANNEL: 2}
    _module_type = MODBUS_TYPE


class modbus_channel_status(__Modbus):
    def exec(self): return self._send_cmd(rcommands.QueryChannelConfig, self.channel).value


class modbus_reset_channel(__Modbus):
    def exec(self): return self._send_cmd(rcommands.ResetChannel, self.channel)


class modbus_set_channel_conf(__Modbus):
    opt_params = {BAUD: 2400, PARITY: 'odd', STOP_BITS: 2}

    def exec(self):
        try:
            baud = rresp.ChannelConfig._values["baud"].index(self.next_param)
            parity = rresp.ChannelConfig._values["parity"].index(self.next_param)
            stop = rresp.ChannelConfig._values["stop"].index(self.next_param)
        except:
            raise BadParameter()

        return self._send_cmd(rcommands.SetChannelConfig, self.channel, baud, parity, stop)


""" Wizards """


class monitor(Module, groups.Wizards):
    def_params = {}
    var_params = {MODULE: 1, CHANNEL: 1, DEVICE_ADDR: 'D3,-D4', INSTANCE: 'IT4', NAME: 'event'}

    @classmethod
    def example(cls):
        return ' '.join([cls.domain_name, cls.__name__] +
                        [f'{cls.def_params[p]}' for p in cls.def_params] +
                        [f'{cls.opt_params[p]}' for p in cls.opt_params] +
                        [f'{cls.var_params[p]}' for p in cls.var_params]).lower() + \
               '\n' + cls.loc(cls.domain_name, cls.__name__, NAMED_PARAMS_DESC) + \
               '\n' + cls.loc(cls.domain_name, cls.__name__, EXCLUDE_PARAMS_DESC) + \
               '\n' + cls.loc(cls.domain_name, cls.__name__, BROADCAST_VALUE)

    def _prepare(self):
        self.filter = monitor_filter.Filter(self._args)
        self.can = CanConnection(interface=settings.conf.interface)

    def exec(self):
        try:
            self._prepare()
        except OSError as ex:
            raise CanOpenError(reason=str(ex))

        try:
            for dump in self.can.bus:
                dump = CanDump.can_dump(dump)
                try:
                    payload = dump.payload
                    if self.filter.check(payload):
                        log.monitor(payload)
                except BaseException as ex:
                    try:
                        log.exception(PacketParseError(dump.payload.header_str() + ' ' + str(ex)))
                    except:
                        log.exception(PacketParseError(tools.bytes2str(dump) + ' ' + str(ex)))
        except:
            return

    def set_break(self, sig, params):
        super(monitor, self).set_break(sig, params)
        self.can.stop()
